#include <iostream>
#include <o2scl/sn_fermion.h>
#include <o2scl/eff_fermion.h>
#include <o2scl/rel_fermion.h>
#include <o2scl/sn_classical.h>
#include <o2scl/classical.h>

using namespace std;
using namespace o2scl;

// Things to check. Ensure that
// calc_mu() and calc_density() both work with a bad guess
// The fractional error in sn_fermion is sufficiently small
// sn_fermion matches eff_fermion and rel_fermion
// zero temperature works
// classical matches sn_classical
//
// Remember we're testing performance not correctness


class part_test {
public:

  classical cl;
  sn_classical sc;
  sn_fermion sf;
  eff_fermion ef;
  rel_fermion rf;

  part_test() {
    cl.init(0.0,2.0);
    sc.init(0.0,2.0);
    sf.init(0.0,2.0);
    ef.init(0.0,2.0);
    rf.init(0.0,2.0);
  }

  int copy(part &src, part &dest) {
    dest.n=src.n;
    dest.mu=src.mu;
    dest.ed=src.ed;
    dest.pr=src.pr;
    dest.en=src.en;
    return 0;
  }
  
  int compare(part &one, part &two, double *val) {
    if (fabs(one.n)>1.0e-99) {
      val[0]=fabs(one.n-two.n)/fabs(one.n);
    } else val[0]=-1.0;
    if (fabs(one.mu)>1.0e-99) {
      val[1]=fabs(one.mu-two.mu)/fabs(one.mu);
    } else val[1]=-1.0;
    if (fabs(one.ed)>1.0e-99) {
      val[2]=fabs(one.ed-two.ed)/fabs(one.ed);
    } else val[2]=-1.0;
    if (fabs(one.pr)>1.0e-99) {
      val[3]=fabs(one.pr-two.pr)/fabs(one.pr);
    } else val[3]=-1.0;
    if (fabs(one.en)>1.0e-99) {
      val[4]=fabs(one.en-two.en)/fabs(one.en);
    } else val[4]=-1.0;
    return 0;
  }
  
  int test(double T, double psi, double mot, double val[]) {
    int ret, ix=0;
    fermion temp;

    ef.m=T*mot;
    ef.mu=psi*T+sf.m;
    ret=ef.calc_mu(T);
    copy(ef,temp);
    val[ix++]=ret;
    ret=ef.calc_density(T);
    compare(temp,ef,&val[ix]); 
    ix+=5;
    val[ix++]=ret;
    ef.nu=-1.0e6;
    ret=ef.calc_density(T);
    val[ix++]=ret;
    ef.nu=1.0e6;
    ret=ef.calc_density(T);
    val[ix++]=ret;

    rf.m=T*mot;
    rf.mu=psi*T+sf.m;
    ret=rf.calc_mu(T);
    compare(temp,rf,&val[ix]);
    ix+=5;
    val[ix++]=ret;
    ret=rf.calc_density(T);
    compare(temp,rf,&val[ix]);
    ix+=5;
    val[ix++]=ret;
    rf.nu=-1.0e6;
    ret=rf.calc_density(T);
    val[ix++]=ret;
    rf.nu=1.0e6;
    ret=rf.calc_density(T);
    val[ix++]=ret;

    sf.m=T*mot;
    sf.mu=psi*T+sf.m;
    ret=sf.calc_mu(T);
    compare(temp,sf,&val[ix]);
    val[ix++]=ret;
    val[ix++]=fabs(sf.unc.n/sf.n);
    val[ix++]=fabs(sf.unc.mu/sf.mu);
    val[ix++]=fabs(sf.unc.ed/sf.ed);
    val[ix++]=fabs(sf.unc.pr/sf.pr);
    val[ix++]=fabs(sf.unc.en/sf.en);
    ret=sf.calc_density(T);
    compare(temp,sf,&val[ix]);
    val[ix++]=ret;
    sf.nu=-1.0e6;
    ret=sf.calc_density(T);
    val[ix++]=ret;
    sf.nu=1.0e6;
    ret=sf.calc_density(T);
    val[ix++]=ret;

    return 0;
  }
};

int main(void) {

  cout.setf(ios::scientific);

  part_test pt;
  double vals[100];
  
  for(double T=1.0e-3;T<=1.01e3;T*=1.0e3) {
    double psi=-20.0;
    while (psi<800.0) {
      for(double mot=1.0e-5;mot<=1.01e3;mot*=10.0) {
	pt.test(T,psi,mot,vals);
	cout << T << " " << psi << " " << mot << " " << vals[8] << endl;
      }
      if (psi>0.0) psi*=4.0;
      if (psi<0.0) psi/=4.0;
      if (psi>-1.0 && psi<0.0) psi*=-1.0;
    }
  }

  return 0;
}

