#include <o2scl/collection.h>
#include <o2scl/file_detect.h>
#include <o2scl/base_ioc.h>

using namespace std;
using namespace o2scl;

int main(void) {
  cout.setf(ios::scientific);

  /// Some data
  char c='t';
  double d=sqrt(2.0);
  int i=10;
  string s="string test";

  /// Make sure the I/O objects are present
  base_ioc bio;

  /// We create a collection, and add all of the data 
  /// items to the collection
  collection co;

  co.add("char_test","char",&c);
  co.add("double_test","double",&d);
  co.add("int_test","int",&i);
  co.add("string_test","string",&s);
  
  /// In order to output things to stdout later
  text_out_file tout(&cout);

  /// Output the data in the collection to several 
  /// different types of files
  
  co.fout("ex_file_1");
  
  text_out_file *tof2=new text_out_file("ex_file_2.gz");
  co.fout(tof2);
  delete tof2;

  binary_out_file *bof=new binary_out_file("ex_file_3");
  co.fout(bof);
  delete bof;

  binary_out_file *bof2=new binary_out_file("ex_file_4.gz");
  co.fout(bof2);
  delete bof2;

  /// Erase all of the information in the collection
  co.clear();
  
  /// Now read in each of the files

  file_detect fd;

  cout.precision(10);

  cout << "ex_file_1:\n" << endl;
  co.fin(fd.open("ex_file_1"));
  if (fd.is_compressed()) cout << "Compressed." << endl;
  if (fd.is_binary()) cout << "Binary." << endl;
  fd.close();
  co.fout(&tout);
  co.clear();

  cout << "\nex_file_2:\n" << endl;
  co.fin(fd.open("ex_file_2.gz"));
  if (fd.is_compressed()) cout << "Compressed." << endl;
  if (fd.is_binary()) cout << "Binary." << endl;
  fd.close();
  co.fout(&tout);
  co.clear();

  cout << "\nex_file_3:\n" << endl;
  co.fin(fd.open("ex_file_3"));
  if (fd.is_compressed()) cout << "Compressed." << endl;
  if (fd.is_binary()) cout << "Binary." << endl;
  fd.close();
  co.fout(&tout);
  co.clear();
  
  cout << "\nex_file_4:\n" << endl;
  co.fin(fd.open("ex_file_4.gz"));
  if (fd.is_compressed()) cout << "Compressed." << endl;
  if (fd.is_binary()) cout << "Binary." << endl;
  fd.close();
  co.fout(&tout);
  co.clear();

  return 0;
}
