#include <iostream>
#include <o2scl/base_ioc.h>
#include <o2scl/table.h>
#include <o2scl/collection.h>
#include <o2scl/constants.h>

using namespace std;
using namespace o2scl;

int main(void) {

  cout.setf(ios::scientific);
  
  // Create a table with two columns. It is usually best to
  // specify in the constructor, the number of rows you will 
  // need before hand, i.e. table dat(100), but we do not
  // do so here for demonstration purposes.
  table dat;
  dat.line_of_names("x y");
  
  for(int i=0;i<11;i++) {
    dat.set("x",i,((double)i)/10.0);
    dat.set("y",i,sin(dat.get("x",i)));
  }
  
  // We demonstrate interpolation and differentiation and 
  // compare with the exact answers
  cout << " Result        Expected" << endl;
  cout << "----------------------------" << endl;
  cout << " " << dat.interp("y",0.5,"x") << "  " << asin(0.5) << endl;
  cout << " " << dat.deriv("x",0.5,"y") << "  " << cos(0.5) << endl;
  cout << dat.deriv2("x",0.5,"y") << " " << -sin(0.5) << endl;
  cout << endl;

  // We can also create columns from algebraic formulas.
  dat.make_cols("cx=sqrt(1.0-sx*sx) sx=y");

  // We can add "constants" for use in formulas
  dat.add_const("pi",acos(-1.0));
  dat.make_col("sin(x*pi/2)","s2");

  // We get approximate derivatives for an entire column
  dat.deriv("x","sx","cx2");

  base_ioc bio;

  // We output this table to std::cout and also to a file
  collection co;
  cout.setf(ios::scientific);
  text_out_file tout(&cout);
  co.out_one(&tout,"table","dat",&dat);
  table_io.out_one("ex_table.dat","dat",&dat);
  
  return 0;
}
