/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/tensor.h>
#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;

int main(void) {

  test_mgr t;
  t.set_output_level(1);

  {
    tensor_br1  arr;
    size_t *i1=new size_t[1];
    i1[0]=3;
    arr.tensor_allocate(1,i1);
    for(size_t i=0;i<i1[0];i++) {
      arr[i]=((double)i);
      cout << i << " " << arr[i] << endl;
    }
    cout << arr.total_size() << endl;
    arr.tensor_free();
    delete[] i1;
    cout << endl;

    tensor_br<tensor_br1  > mat;
    size_t *i2=new size_t[2];
    i2[0]=4;
    i2[1]=3;
    mat.tensor_allocate(2,i2);
    for(size_t i=0;i<i2[0];i++) {
      for(size_t j=0;j<i2[1];j++) {
	mat[i][j]=((double)i)+j;
	cout << i << " " << j << " " << mat[i][j] << endl;
      }
    }
    cout << mat.total_size() << endl;
    mat.tensor_free();
    delete[] i2;
    cout << endl;

    tensor_br<tensor_br<tensor_br1  > > m3;
    size_t *i3=new size_t[3];
    i3[0]=4;
    i3[1]=3;
    i3[2]=3;
    m3.tensor_allocate(3,i3);
    for(size_t i=0;i<i3[0];i++) {
      for(size_t j=0;j<i3[1];j++) {
	for(size_t k=0;k<i3[2];k++) {
	  m3[i][j][k]=((double)i)+j+k;
	  cout << i << " " << j << " " << k << " "
	       << m3[i][j][k] << endl;
	}
      }
    }
    cout << m3.total_size() << endl;
    cout << endl;

    size_t *i4=new size_t[3];
    for(size_t i=0;i<i3[0];i++) {
      for(size_t j=0;j<i3[1];j++) {
	for(size_t k=0;k<i3[2];k++) {
	  i4[0]=i;
	  i4[1]=j;
	  i4[2]=k;
	  size_t *i5=i4+1;
	  cout << i << " " << j << " " << k << " "
	       << m3.get(i4) << " "
	       << m3[i4[0]].get(i5) << " "
	       << m3[i4[0]].get(i4,1) << " "
	       << endl;
	}
      }
    }

    cout.setf(ios::scientific);
    double **grid=new double *[3];
    grid[0]=new double[4];
    grid[1]=new double[3];
    grid[2]=new double[3];
    grid[0][0]=1.0;
    grid[0][1]=2.0;
    grid[0][2]=3.0;
    grid[0][3]=4.0;
    grid[1][0]=1.0;
    grid[1][1]=2.0;
    grid[1][2]=3.0;
    grid[2][0]=1.0;
    grid[2][1]=2.0;
    grid[2][2]=3.0;
    m3.set_grid(grid);
    for(size_t i=0;i<i3[0];i++) {
      for(size_t j=0;j<i3[1];j++) {
	for(size_t k=0;k<i3[2];k++) {
	  double x=grid[0][i];
	  double y=grid[1][j];
	  double z=grid[2][k];
	  m3[i][j][k]=2.0*x*x-y-3.0*z*z;
	}
      }
    }

    double vals[3]={2.5,2.5,1.5};
    cout << "Int: " << (2.0*2.5*2.5-2.5-3.0*1.5*1.5) << " "
	 << m3.interpolate(grid,vals,i3) << endl;
    double vals2[3]={2.0,2.0,1.0};
    cout << "Int: " << (2.0*2.0*2.0-2.0-3.0*1.0*1.0) << " "
	 << m3.interpolate(grid,vals2,i3) << endl;

    cout << m3.get_size(0) << " " << m3.get_size(1) << " "
	 << m3.get_size(2) << " " << m3.get_size(3) << endl;
    
    m3.tensor_free();
    delete[] i3;
    delete[] i4;

  }

  {
    tensor arr;
    size_t i1[1], j1[1];
    i1[0]=3;
    arr.tensor_allocate(1,i1);
    for(size_t i=0;i<i1[0];i++) {
      j1[0]=i;
      arr.set(j1,((double)i));
      cout << i << " " << arr.get(j1) << endl;
    }
    cout << arr.total_size() << endl;
    arr.tensor_free();
    cout << endl;

    tensor mat;
    size_t i2[2], j2[2];
    i2[0]=4;
    i2[1]=3;
    mat.tensor_allocate(2,i2);
    for(size_t i=0;i<i2[0];i++) {
      for(size_t j=0;j<i2[1];j++) {
	j2[0]=i;
	j2[1]=j;
	mat.set(j2,((double)i)+j);
	cout << i << " " << j << " " << mat.get(j2) << endl;
      }
    }
    cout << mat.total_size() << endl;
    mat.tensor_free();
    cout << endl;
    
    tensor_grid<> m3;
    size_t i3[3], j3[3];
    i3[0]=4;
    i3[1]=3;
    i3[2]=3;
    m3.tensor_allocate(3,i3);
    for(size_t i=0;i<i3[0];i++) {
      for(size_t j=0;j<i3[1];j++) {
	for(size_t k=0;k<i3[2];k++) {
	  j3[0]=i;
	  j3[1]=j;
	  j3[2]=k;
	  m3.set(j3,((double)i)+j+k);
	  cout << i << " " << j << " " << k << " " << m3.get(j3) << endl;
	}
      }
    }
    cout << m3.total_size() << endl;
    cout << endl;

    cout.setf(ios::scientific);
    double **grid=new double *[3];
    size_t j4[3];
    grid[0]=new double[4];
    grid[1]=new double[3];
    grid[2]=new double[3];
    grid[0][0]=1.0;
    grid[0][1]=2.0;
    grid[0][2]=3.0;
    grid[0][3]=4.0;
    grid[1][0]=1.0;
    grid[1][1]=2.0;
    grid[1][2]=3.0;
    grid[2][0]=1.0;
    grid[2][1]=2.0;
    grid[2][2]=3.0;
    m3.set_grid(grid);
    for(size_t i=0;i<i3[0];i++) {
      for(size_t j=0;j<i3[1];j++) {
	for(size_t k=0;k<i3[2];k++) {
	  double x=grid[0][i];
	  double y=grid[1][j];
	  double z=grid[2][k];
	  j3[0]=i;
	  j3[1]=j;
	  j3[2]=k;
	  m3.set(j3,2.0*x*x-y-3.0*z*z);
	  size_t ix;
	  ix=m3.pack_indices(j3);
	  m3.unpack_indices(ix,j4);
	  t.test_gen(j4[0]==i,"pack/unpack 1");
	  t.test_gen(j4[1]==j,"pack/unpack 2");
	  t.test_gen(j4[2]==k,"pack/unpack 3");
	}
      }
    }
    
    t.test_gen(m3.get_size(0)==4,"size 1");
    t.test_gen(m3.get_size(1)==3,"size 2");
    t.test_gen(m3.get_size(2)==3,"size 3");

    double vals[3]={2.5,2.5,1.5};
    cout << "Int: " << (2.0*2.5*2.5-2.5-3.0*1.5*1.5) << " "
	 << m3.interpolate(vals) << endl;
    double vals2[3]={2.0,2.0,1.0};
    cout << "Int: " << (2.0*2.0*2.0-2.0-3.0*1.0*1.0) << " "
	 << m3.interpolate(vals) << endl;

    m3.tensor_free();

  }
  
  t.report();

  return 0;
}
