/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_GEN_INTE_H
#define O2SCL_GEN_INTE_H

#ifndef DOXYGENP
namespace o2scl {
#endif
  
  /** 
      \brief Generalized multi-dimensional integration base
 
      In order to allow the user to specify only three functions (for
      the integrand, the lower limits, and the upper limits) the first
      integer variable is used to distinguish among the variable
      limits. So the function \f$ a_0() \f$ is just lower(0,NULL,vp)
      where vp is a void pointer, the function \f$ a_1 \f$ is
      lower(1,x,vp) where x is a 1-dimensional vector, and the
      function \f$ a_i \f$ is lower(i,x,vp) where x is an
      i-dimensional vector. Similarly, the function \f$ b_i \f$ is
      upper(i,x,vp) where x is an i-dimensional vector.
  */
  template<class param_t, class func_t, class lfunc_t, class ufunc_t,
    class vec_t=ovector_view> class gen_inte {

  
      public:
  
      /** \brief Verbosity
       */
      int verbose;
      
      /** \brief The maximum "uncertainty" in the value of the integral
       */
      double tolf;
  
      gen_inte() 
      {
	tolf=1.0e-8;
	verbose=0;
	interror=0.0;
      }
  
      /** \brief Integrate function \c func from
	  \f$ x_i=f_i(x_i) \f$ to \f$ x_i=g_i(x_i) \f$ for
	  \f$ 0<i<\mathrm{ndim}-1 \f$.
 
      */
      double ginteg(func_t &func, size_t ndim, lfunc_t &a, ufunc_t &b,
		    param_t &pa) 
      {
	return 0;
      }
  
      /** \brief Integrate function \c func from
	  \f$ x_i=f_i(x_i) \f$ to \f$ x_i=g_i(x_i) \f$ for
	  \f$ 0<i<\mathrm{ndim}-1 \f$.
 
      */
      int ginteg_err(func_t &func, size_t ndim, lfunc_t &a, ufunc_t &b, 
		     param_t &pa, double &res, double &err)
      {
	return 0;
      }
  
      /** \brief Return the error in the result from the last call to 
	  ginteg() or ginteg_err()
      
	  This will quietly return zero if no integrations have been
	  performed.
      */
      double get_error() { return interror; }
  
      /// Return string denoting type ("gen_inte")
      const char *type() { return "gen_inte"; }
  
#ifndef DOXYGEN_INTERNAL
  
      protected:
  
      /// The uncertainty for the last integration computation
      double interror;
  
#endif
  
    };

#ifndef DOXYGENP
}
#endif

#endif





  
