/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/test_mgr.h>
#include <o2scl/multi_funct.h>
#include <o2scl/naive_metropolis.h>
#include <o2scl/gsl_inte_qng.h>

using namespace std;
using namespace o2scl;

double dist[10], gbeta=1.0;

double ener(size_t nv, const ovector_view &x, void *&vp) {
  return x[0]*x[0];
}

double ener1d(double x, void *&vp) {
  return exp(-x*x*gbeta);
}

double f(size_t nv, const ovector_view &x, void *&vp) {
  dist[(int)(x[0]*10.0)]++;
  return x[0];
}

double fener1d(double x, void *&vp) {
  return x*exp(-x*x*gbeta);
}

int main(void) {
  test_mgr t;
  t.set_output_level(2);

  cout.setf(ios::scientific);

  multi_funct_fptr_noerr<void *> mener(ener);
  multi_funct_fptr_noerr<void *> mf(f);

  naive_metropolis<void *,multi_funct<void *>,gsl_rnga,ovector_view,
    ofvector<1> > nm;

  ovector a(1), b(1);
  a[0]=0.0;
  b[0]=1.0;
  double val, err, exact;
  void *vpx=0;
  
  nm.niter=10000;
  nm.minteg_err(mf,mener,1,a,b,gbeta,val,err,vpx);
  cout << "MC result: " << val << " " << err << endl;
  cout << endl;
  
  cout << "Output distribution: " << endl;
  for(int i=0;i<10;i++) {
    cout << dist[i] << " ";
    if (i==4) cout << endl;
  }
  cout << endl;
  cout << endl;

  gsl_inte_qng<void *,funct<void *> > ge;
  funct_fptr_noerr<void *> mener1d(ener1d);
  funct_fptr_noerr<void *> mfener1d(fener1d);
  
  cout << "Show distribution approximates the integrand of the " <<
    "denominator:" << endl;
  for(int i=0;i<10;i++) {
    double p=ge.integ(mener1d,((double)i)/10.0,((double)(i+1))/10.0,vpx)/
      ge.integ(mener1d,0.0,1.0,vpx);
    cout << ((int)(dist[i]/p)) << " ";
    if (i==4) cout << endl;
  }
  cout << endl;
  cout << endl;
  
  cout << "Exact result: " << ge.integ(mfener1d,0.0,1.0,vpx)/
    ge.integ(mener1d,0.0,1.0,vpx) << endl;
  cout << endl;
  
  cout << "Compare results at different temperatures: " << endl;
  for(int k=0;k<20;k++) {
    gbeta=1.0*pow(4.0,((double)k-10));
    //  gbeta=1.0;
  
    nm.niter=1000;
    nm.minteg_err(mf,mener,1,a,b,gbeta,val,err,vpx);
  
    exact=ge.integ(mfener1d,0.0,1.0,vpx)/
      ge.integ(mener1d,0.0,1.0,vpx);
  
    cout << gbeta << " ";
    cout << exact << " ";
    cout << val << " " << err/val << " ";
    cout << (val-exact)/exact << endl;

  }
  cout << endl;
  
  cout << "Show that more iterations fixes problems with small T: " << endl;
  nm.niter=100000;
  nm.minteg_err(mf,mener,1,a,b,gbeta,val,err,vpx);
  
  exact=ge.integ(mfener1d,0.0,1.0,vpx)/
    ge.integ(mener1d,0.0,1.0,vpx);
  
  cout << gbeta << " ";
  cout << exact << " ";
  cout << val << " " << err/val << " ";
  cout << (val-exact)/exact << endl;
  cout << endl;

  t.report();
  return 0;

}
