/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include "nuclear_dist.h"

using namespace std;
using namespace o2scl;

#include <o2scl/constants.h>

using namespace o2scl_const;

int main(void) {
  test_mgr t;
  t.set_output_level(2);
   
  cout.setf(ios::scientific);
  
  mnms95_mass mth;
  ame_mass ame;
  mnms95_mass_exp mexp;
  
  simple_dist sd(3,6,10,12,&mth);
  for(nuclear_dist::iterator ndi=sd.begin();ndi!=sd.end();ndi++) {
    cout << ndi->Z << " " << ndi->A << " " << ndi->m << endl;
  }
  t.test_gen(sd.size()==12,"sd size");
  
  simple_dist sd2(82,82,208,208,&mth);
  nuclear_dist::iterator ndi2=sd2.begin();
  cout << ndi2->Z << " " << ndi2->A << " " << ndi2->m*hc_mev_fm << endl;
  cout << ndi2->mex*hc_mev_fm << " " << ndi2->be/208.0*hc_mev_fm << endl;
  t.test_rel(ndi2->mex*hc_mev_fm,-21.749,5.0e-2,"Pb mex");
  t.test_rel(ndi2->be/208.0*hc_mev_fm,-7.867,5.0e-4,"Pb be");
  
  full_dist fd(&mth);
  t.test_gen(fd.size()==8979,"full_dist size 1");
  
  full_dist fd2(&ame);
  t.test_gen(fd2.size()==3178,"full_dist size 2");
 
  full_dist fd3;
  fd3.set_dist(&ame);
  t.test_gen(fd3.size()==3178,"full_dist size 3");

  full_dist fd4;
  fd4.set_dist(&mexp);
  t.test_gen(fd4.size()==1654,"full_dist size 4");
  
  t.report();
  return 0;
}
