/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/test_mgr.h>
#include <o2scl/ode_bv_solve.h>
#include <o2scl/ode_funct.h>

using namespace std;
using namespace o2scl;

/* As an example, we solve y''[x]==-y[x] with the boundary
   conditions y[0]==1 and y'[1]==2. The solution is
   y=cos(x)+(2.0*sec(1)+tan(1))*sin(x)
*/
 
int derivs(double x, size_t nv, const ovector_view &y, ovector_view &dydx,
	   void *&vp) {
  dydx[0]=y[1];
  dydx[1]=-y[0];
  return 0;
}

int exact_sol(double x, double &y, double &dydx) {
  y=cos(x)+(2.0/cos(1.0)+tan(1.0))*sin(x);
  dydx=-sin(x)+(2.0/cos(1.0)+tan(1.0))*cos(x);
  return 0;
}

int main(void) {
  test_mgr t;
  t.set_output_level(1);
  
  cout.setf(ios::scientific);

  ode_bv_shoot<void *,ode_funct<void *> > obs;

  ode_funct_fptr<void *> off(derivs);
  double x0=0.0, x1=1.0, y, dydx;
  void *vp=0;
  ovector ystart(2), yend(2);
  ovector_int index(2);
  ystart[0]=1.0;
  ystart[1]=0.0;
  yend[0]=0.0;
  yend[1]=2.0;
  index[0]=ode_bv_solve<void *,ode_funct<void *> >::left;
  index[1]=ode_bv_solve<void *,ode_funct<void *> >::right;
  
  obs.solve(x0,x1,(x1-x0)/20.0,2,ystart,yend,index,vp,off);

  cout << ystart[0] << " " << ystart[1] << endl;
  cout << yend[0] << " " << yend[1] << endl;
  cout << endl;
  
  exact_sol(x0,y,dydx);
  cout << y << " " << dydx << endl;
  t.test_rel(ystart[1],dydx,1.0e-6,"ystart[1]");
  exact_sol(x1,y,dydx);
  cout << y << " " << dydx << endl;
  t.test_rel(yend[0],y,1.0e-6,"yend[0]");
  t.test_rel(yend[1],dydx,1.0e-6,"yend[1]");
  cout << endl;
  
  t.report();
  return 0;
}

