/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/eff_boson.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

#include <o2scl/gsl_quad.h>
#include <o2scl/rel_boson.h>
#include <o2scl/gsl_mroot_hybrids.h>

int main(void) {
  test_mgr t;
  t.set_output_level(2);

  eff_boson b(1.0,2.0);
  t.test_rel(b.m,1.0,1.0e-6,"mass inheritance");
  gsl_mroot_hybrids gs, gs2, gs3;
  cout.setf(ios::scientific);
  b.non_interacting=true;
  eff_boson::loadcoeff("bosejel21");

  double T;
  int ret;

  // -----------------------------------------------------------------
  // eff_boson tests

  t.set_output_level(1);

  cout << "----------------------------------------------------" << endl;
  cout << "eff_boson tests:" << endl;
  cout << "----------------------------------------------------" << endl;
  cout << endl;

  cout << "massless, non-degenerate (mu=m=0) "
       << "(m=0.011, mu=0.01, T=0.1)" << endl;
  b.m=0.011;
  b.mu=0.01;
  T=0.1;
  cout << "calc_mu(T) vs. calc_density(T) vs. massless_calc_mu(T)" << endl;
  ret=b.calc_mu(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl; 
  double tn=b.n, ted=b.ed, tpr=b.pr, ten=b.en;
  ret=b.calc_density(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl;
  t.test_rel(b.n,tn,2.0e-6,"density");
  t.test_rel(b.ed,ted,2.0e-6,"energy");
  t.test_rel(b.pr,tpr,2.0e-6,"pressure");
  t.test_rel(b.en,ten,2.0e-6,"entropy");
  b.m=0.011;
  b.mu=0.01;
  T=0.1;
  ret=b.massless_calc_mu(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl;
  t.test_rel(b.n,tn,2.0e-1,"density");
  t.test_rel(b.ed,ted,2.0e-1,"energy");
  t.test_rel(b.pr,tpr,2.0e-1,"pressure");
  t.test_rel(b.en,ten,2.0e-1,"entropy");
  cout << endl;

  cout << "degenerate relativistic (mu=m, mu>>T) "
       << "(m=0.11, mu=0.1, T=0.01)" << endl;
  b.m=0.11;
  b.mu=0.1;
  T=0.01;
  cout << "calc_mu(T) vs. calc_density(T)" << endl;
  ret=b.calc_mu(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl;
  tn=b.n, ted=b.ed, tpr=b.pr, ten=b.en;
  ret=b.calc_density(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl;
  cout << endl;
  t.test_rel(b.n,tn,2.0e-6,"density");
  t.test_rel(b.ed,ted,2.0e-6,"energy");
  t.test_rel(b.pr,tpr,2.0e-6,"pressure");
  t.test_rel(b.en,ten,2.0e-5,"entropy");

  cout << "Testing mu=m limit "
       << "(m=0.101, mu=0.1, T=0.01)" << endl;
  b.m=0.101;
  b.mu=0.1;
  T=0.01;
  cout << "calc_mu(T) vs. calc_density(T)" << endl;
  ret=b.calc_mu(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl;
  cout << "(m=0.1, mu=0.1, T=0.01)" << endl;
  b.m=0.1;
  b.mu=0.1;
  T=0.01;
  cout << "calc_mu(T) vs. calc_density(T)" << endl;
  ret=b.calc_mu(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl;
  cout << endl;

  cout << "degenerate non-relativistic (m>>mu>>T) "
       << "(m=0.5, mu=0.1, T=0.05)" << endl;
  b.m=0.5;
  b.mu=0.1;
  T=0.05;
  cout << "calc_mu(T) vs. calc_density(T)" << endl;
  ret=b.calc_mu(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl;
  ret=b.calc_density(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl;
  cout << endl;

  cout << "non-degenerate relativistic (mu=m, mu<<T) "
       << "(m=0.011, mu=0.01, T=0.1)" << endl;
  b.m=0.011;
  b.mu=0.01;
  T=0.1;
  cout << "calc_mu(T) vs. calc_density(T)" << endl;
  ret=b.calc_mu(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl;
  ret=b.calc_density(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl;
  cout << "pair_mu(T) vs. pair_density(T)" << endl;
  b.m=0.011;
  b.mu=0.01;
  T=0.1;
  ret=b.pair_mu(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl;
  tn=b.n, ted=b.ed, tpr=b.pr, ten=b.en;
  ret=b.pair_density(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl;
  cout << endl;
  //  t.test_rel(b.n,tn,2.0e-6,"density");
  //  t.test_rel(b.ed,ted,2.0e-6,"energy");
  //  t.test_rel(b.pr,tpr,2.0e-6,"pressure");
  //  t.test_rel(b.en,ten,2.0e-6,"entropy");

  cout << "non-degenerate non-relativistic (mu<<m, mu<<T) "
       << "(m=0.1, mu=0.01, T=1.0)" << endl;
  b.m=0.1;
  b.mu=0.01;
  T=1.0;
  cout << "calc_mu(T) vs. calc_density(T)" << endl;
  ret=b.calc_mu(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl; 
  tn=b.n, ted=b.ed, tpr=b.pr, ten=b.en;
  ret=b.calc_density(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl;
  t.test_rel(b.n,tn,2.0e-6,"density");
  t.test_rel(b.ed,ted,2.0e-6,"energy");
  t.test_rel(b.pr,tpr,2.0e-6,"pressure");
  t.test_rel(b.en,ten,2.0e-6,"entropy");
  cout << "pair_mu(T) vs. pair_density(T)" << endl;
  b.m=0.1;
  b.mu=0.01;
  T=1.0;
  ret=b.pair_mu(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl;
  tn=b.n, ted=b.ed, tpr=b.pr, ten=b.en;
  ret=b.pair_density(T);
  cout << ret << " " << b.n << " " << b.mu << " " << b.ed << " " 
       << b.pr << " " << b.en << endl;
  cout << endl;
  //  t.test_rel(b.n,tn,2.0e-6,"density");
  //  t.test_rel(b.ed,ted,2.0e-6,"energy");
  //  t.test_rel(b.pr,tpr,2.0e-6,"pressure");
  //  t.test_rel(b.en,ten,2.0e-6,"entropy");

  // -----------------------------------------------------------------
  // eff_boson coefficients
  
  cout << "----------------------------------------------------" << endl;
  cout << "eff_boson coefficients:" << endl;
  cout << "----------------------------------------------------" << endl;
  cout << endl;

  gsl_inte_qagiu nit;
  gsl_inte_qag dit;
  rel_boson b3(1.0,2.0);
  b3.set_inte(nit,dit);

  double little_t, psi;
  int ret1, ret2;
  for(little_t=0.05;little_t<=101;little_t*=2.0) {
    for(psi=-2.0/little_t;psi<=0.01/little_t;psi+=1.0/3.0/little_t) {
      if (psi*little_t>-0.01) psi=0.0;
      T=1.0;
      b.m=T*little_t;
      b.mu=psi*T+b.m;
      b3.m=b.m;
      b3.mu=b.mu;
      ret1=b.calc_mu(T);
      ret2=b3.calc_mu(T);
      cout << ret1 << " " << ret2 << " " << little_t << " " 
	   << psi*little_t << " " << b.en << " " << b3.en << endl;
    }
    if (little_t>0.09 && little_t<0.11) little_t*=2.5;
    if (little_t>0.9 && little_t<1.1) little_t*=2.5;
    if (little_t>9.0) little_t*=5.0;
  }
  
  t.report();
  return 0;
}

