/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_EFF_QUARK_H
#define O2SCL_EFF_QUARK_H

#include <string>
#include <iostream>
#include <fstream>
#include <cmath>
#include <o2scl/constants.h>
#include <o2scl/mroot.h>
#include <o2scl/inte.h>
#include <o2scl/eff_fermion.h>
#include <o2scl/quark.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Quark class from fitting method

      \todo Add testing.
  */
  class eff_quark : public eff_fermion, public quark {

  public:

    /// Create a quark with mass \c m  and degeneracy \c g 
    eff_quark(double m=0.0, double g=0.0);

    virtual ~eff_quark();

    virtual int calc_mu(const double temper);
    virtual int calc_density(const double temper);
    virtual int pair_mu(const double temper);
    virtual int pair_density(const double temper);
    
    friend class io_tlate<eff_quark>;

    /// Return string denoting type ("eff_quark")
    virtual const char *type() { return "eff_quark"; }

  protected:

  };

  template<> int io_tlate<eff_quark>::input
    (cinput *co, in_file_format *ins, eff_quark *q);
  template<> int io_tlate<eff_quark>::output
    (coutput *co, out_file_format *outs, eff_quark *q);
  template<> const char *io_tlate<eff_quark>::type();
  
  typedef io_tlate<eff_quark> eff_quark_io_type;

#ifndef DOXYGENP
}
#endif

#endif
